#include <glib.h>
#include <gtk/gtk.h>
#include <gdk/gdk.h>

#ifdef HAVE_CONFIG_H
# include "../grr_config.h"
#endif

#include "internal.h"

#include "conversation.h"
#include "debug.h"
#include "gtkconv.h"
#include "gtkplugin.h"
#include "gtkprefs.h"
#include "gtkutils.h"
#include "prefs.h"
#include "notify.h"

#include "channel.h"
#include "channels.h"
#include "grr_prefs.h"
#include "version.h"

static GList *buttons;
static GtkIconFactory *icon_factory;

static GtkWidget *pref_tree;
// static GtkTreeModel *model;
static GtkListStore *model;

static GtkWidget *name_entry;
static GtkWidget *url_entry;

static void *about_handle=0;


static void grr_menu_about_cb(GtkMenuItem *item, gpointer data)
{
    if(about_handle)
    {
        gaim_notify_close(GAIM_NOTIFY_FORMATTED, about_handle);
        about_handle = 0;
    }
    about_handle = gaim_notify_formatted(NULL, _("RSS-Reader plugin"),
					 _("RSS-Reader plugin"), NULL,
					 "A small plugin that adds a button to the<br>"
					 "Gaim conversation windows, so that you<br>"
					 "can read RSS feeds from within Gaim.<br>"
					 "<br>Website: <a href=\"http://gaim-rss-reader.sourceforge.net\">http://gaim-rss-reader.sourceforge.net</a><br>"
					 ,
                                         NULL, NULL);
}

static void grr_menu_update_cb(GtkMenuItem *item, gpointer data)
{
    update_immediately();
    //g_print("Update\n");
}


static void grr_menu_feed_cb(GtkMenuItem *item, gpointer data)
{
    Channel *c = get_channel_by_url((gchar *)data);

 
    if(c!=NULL)
	gaim_notify_formatted(NULL, _(c->title),
			      c->title, NULL, rss_channel_html(c),
			      NULL, NULL);
}

static GtkWidget *grr_make_menu(GaimConversation *conv)
{
    GtkWidget *menu, *item;
    GList *feeds = NULL;

    menu = gtk_menu_new();

    feeds = gaim_prefs_get_string_list("/plugins/gtk/gaim-rss-reader/feeds");

    if(!feeds)
    {
        item = gaim_new_item_from_stock(menu, "No feeds", 0, NULL, NULL, 0, 0, NULL);
        gtk_widget_set_sensitive(item, FALSE);
    }
    else
    {
        while(feeds!=NULL)
	{
            gchar **ip = g_strsplit(feeds->data, ";", 2);
	    gchar *url = g_strdup(ip[1]);
	    Channel *c = get_channel_by_url(url);
	    item = gaim_new_item_from_stock(menu, ip[0], 0, G_CALLBACK(grr_menu_feed_cb), url, 0, 0, NULL);
            if(c!=NULL)
		gtk_widget_set_sensitive(item, TRUE);
            else
		gtk_widget_set_sensitive(item, FALSE);
            feeds = g_list_next(feeds);
            g_strfreev(ip);
        }
    }

    gaim_separator(menu);

    item = gaim_new_item_from_stock(menu, "Update", 0, G_CALLBACK(grr_menu_update_cb), NULL, 0, 0, NULL);
    gtk_widget_set_sensitive(item, TRUE);

    item = gaim_new_item_from_stock(menu, "About", 0, G_CALLBACK(grr_menu_about_cb), NULL, 0, 0, NULL);
    gtk_widget_set_sensitive(item, TRUE);

    return menu;
}

static void grr_button_clicked_cb(GtkButton *button, gpointer data)
{
    GaimConversation *conv;
    GtkWidget *menu;

    gaim_debug(GAIM_DEBUG_WARNING, "Gaim-RSS-Reader", "RSS button clicked\n");

    conv = g_object_get_data(G_OBJECT(button), "conv");
    menu = grr_make_menu(conv);

//    gtk_widget_grab_focus(GAIM_GTK_CONVERSATION(conv)->entry);

    gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL, 0,
                   gtk_get_current_event_time());
}

static void grr_add_button(GaimConversation *conv)
{
    GaimGtkConversation *gtkconv = GAIM_GTK_CONVERSATION(conv);
    GaimConversationType type = gaim_conversation_get_type(conv);
    GtkWidget *button = NULL;

    if(type != GAIM_CONV_IM && type != GAIM_CONV_CHAT)
        return;

    button = gaim_gtkconv_button_new("grr-xml","RSS","RSS Reader",gtkconv->tooltips,grr_button_clicked_cb,conv);

    buttons = g_list_append(buttons, (gpointer)button);

    gtk_size_group_add_widget(gtkconv->sg, button);
    gtk_box_pack_start(GTK_BOX(gtkconv->bbox), button, FALSE, FALSE, 0);

    gtk_widget_show(button);
}

static void grr_conv_created_cb(GaimConversation *conv, gpointer data)
{
    GaimGtkConversation *gtkconv;

    if((gtkconv = GAIM_GTK_CONVERSATION(conv)) == NULL)
        return;

    grr_add_button(conv);

    //g_signal_connect(G_OBJECT(gtkconv->entry), "populate-popup",
    //                 G_CALLBACK(gxr_popup_cb), (gpointer)conv);
}

static void
    grr_conv_destroyed_cb(GaimConversation *conv, gpointer data)
{
    GaimConversation *stored_conv;
    GtkWidget *button;
    GList *l, *l_next;

    for(l = buttons; l != NULL; l = l_next) {
        l_next = l->next;

        button = GTK_WIDGET(l->data);
        stored_conv = (GaimConversation *)g_object_get_data(G_OBJECT(button), "conv");

        if(stored_conv == conv) {
            gtk_widget_destroy(button);

            buttons = g_list_remove(buttons, l->data);
            break;
        }
    }
}

static gchar *grr_file_name(const gchar *file_name)
{
    return g_build_filename(DATADIR, "pixmaps", "gaim", "grr", file_name,
                            NULL);
}

static void grr_add_to_stock(const gchar *file_name, const gchar *stock_name)
{
    GtkIconSet *icon_set;
    GdkPixbuf *pixbuf;
    gchar *file = grr_file_name(file_name);

    pixbuf = gdk_pixbuf_new_from_file(file, NULL);
    g_free(file);

    icon_set = gtk_icon_set_new_from_pixbuf(pixbuf);
    g_object_unref(G_OBJECT(pixbuf));

    gtk_icon_factory_add(icon_factory, stock_name, icon_set);
    gtk_icon_set_unref(icon_set);
}

static void grr_init_stock()
{
	icon_factory = gtk_icon_factory_new();
	gtk_icon_factory_add_default(icon_factory);

	grr_add_to_stock("xml.png", "grr-xml");
}

static gboolean grr_load(GaimPlugin *plugin)
{
    void *conv_handle = gaim_conversations_get_handle();

    grr_init_stock();
    /* connect to the signals we need.. */
    gaim_signal_connect(conv_handle, "conversation-created", plugin,
                        GAIM_CALLBACK(grr_conv_created_cb), NULL);

    gaim_signal_connect(conv_handle, "deleting-conversation", plugin,
			GAIM_CALLBACK(grr_conv_destroyed_cb), NULL);

    set_timeout(gaim_prefs_get_int("/plugins/gtk/gaim-rss-reader/update_time"));

    read_feed_desc();

    start_updater();

    return TRUE;
}

static gboolean
grr_unload()
{
    g_list_free(buttons);

    /* remove our icons */
    gtk_icon_factory_remove_default(icon_factory);

    return TRUE;
}

static void init_plugin(GaimPlugin *plugin)
{
    gaim_prefs_add_none("/plugins/gtk/gaim-rss-reader");
    gaim_prefs_add_int("/plugins/gtk/gaim-rss-reader/update_time", 60);

    GList *sl = NULL;
    sl = g_list_append(sl, "Debian Planet;http://www.debianplanet.org/debianplanet/backend.php");
    sl = g_list_append(sl, "Mozillazine;http://www.mozillazine.org/contents.rdf");
    sl = g_list_append(sl, "Newsforge;http://www.newsforge.com/newsforge.rss");
    sl = g_list_append(sl, "Footnotes;http://www.gnomedesktop.org/backend2.php");
    

    gaim_prefs_add_string_list("/plugins/gtk/gaim-rss-reader/feeds", sl);

}


static GtkTreeModel *
create_and_fill_model (void)
{
    GtkListStore *store;
    GtkTreeIter  iter;

    GList *feeds = NULL;

    store = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_STRING);

    feeds = gaim_prefs_get_string_list("/plugins/gtk/gaim-rss-reader/feeds");

    if(feeds)
    {
	while(feeds!=NULL)
	{
	    gchar **ip = g_strsplit(feeds->data, ";", 2);
	    //gchar *url = g_strdup(ip[1]);

            gtk_list_store_append(store, &iter);
	    gtk_list_store_set(store, &iter, 0, ip[0], 1, ip[1], -1);

	    feeds = g_list_next(feeds);
	    g_strfreev(ip);
	}
    }

    return GTK_TREE_MODEL(store);
}


static void add_selected_row_to_list(GtkTreeModel *model, GtkTreePath *path,
	GtkTreeIter *iter, gpointer data)
{
	GSList **list = (GSList **)data;
	*list = g_slist_append(*list, gtk_tree_path_copy(path) );
}

static void remove_row(void *data1, gpointer data2)
{
    GtkTreePath *path = (GtkTreePath*)data1;
    GtkTreeIter iter;
    gtk_tree_model_get_iter(GTK_TREE_MODEL(model), &iter, path);
    gtk_list_store_remove(model, &iter);
    gtk_tree_path_free(path);
}

static void list_save()
{
    GtkTreeIter iter;
    GSList *sl = NULL;

    if(gtk_tree_model_get_iter_first(GTK_TREE_MODEL(model), &iter))
    {
	do
	{
	    GValue val0, val1;
	    gchar *tmp;

	    memset(&val0, 0, sizeof(val0));
	    memset(&val1, 0, sizeof(val1));
	    gtk_tree_model_get_value(GTK_TREE_MODEL(model), &iter, 0, &val0);
	    gtk_tree_model_get_value(GTK_TREE_MODEL(model), &iter, 1, &val1);

	    tmp = g_strdup_printf("%s;%s", g_value_get_string(&val0), g_value_get_string(&val1));
	    sl = g_list_append(sl, g_strdup(tmp));

	    //g_print("%s\n", tmp);

	    g_free(tmp);

	    g_value_unset(&val0);
	    g_value_unset(&val1);
	} while(gtk_tree_model_iter_next(GTK_TREE_MODEL(model), &iter));
    }


    gaim_prefs_set_string_list("/plugins/gtk/gaim-rss-reader/feeds", sl);

    read_feed_desc();

    g_slist_free(sl);
}

static void list_add_new()
{
    GtkTreeIter iter;

    gtk_list_store_append(model, &iter);
    gtk_list_store_set(model, &iter,
		       0, gtk_entry_get_text(GTK_ENTRY(name_entry)),
		       1, gtk_entry_get_text(GTK_ENTRY(url_entry)),
                       -1);

    start_single_updater(gtk_entry_get_text(GTK_ENTRY(name_entry)), gtk_entry_get_text(GTK_ENTRY(url_entry)));

    gtk_editable_select_region(GTK_EDITABLE(name_entry), 0, -1);
    gtk_widget_grab_focus(name_entry);
    gtk_entry_set_text(GTK_ENTRY(name_entry), "");
    gtk_entry_set_text(GTK_ENTRY(url_entry), "");

    list_save();
}

static void list_delete()
{
    GtkTreeSelection *sel;
    GSList *list = NULL;

    sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(pref_tree));
    gtk_tree_selection_selected_foreach(sel, add_selected_row_to_list, &list);

    g_slist_foreach(list, remove_row, NULL);
    g_slist_free(list);

    list_save();
}

static void
check_if_something_is_selected(GtkTreeModel *model,
	GtkTreePath *path, GtkTreeIter *iter, gpointer data)
{
	*((gboolean*)data) = TRUE;
}

static void on_selection_changed(GtkTreeSelection *sel,
				 gpointer data)
{
    gboolean is = FALSE;
    gtk_tree_selection_selected_foreach(sel, check_if_something_is_selected, &is);
    gtk_widget_set_sensitive((GtkWidget*)data, is);
}

static gboolean non_empty(const char *s)
{
    while(*s && g_ascii_isspace(*s))
	s++;
    return *s;
}

static void on_entry_changed(GtkEditable *editable, gpointer data)
{
    gtk_widget_set_sensitive((GtkWidget*)data,
			     non_empty(gtk_entry_get_text(GTK_ENTRY(name_entry))) &&
			     non_empty(gtk_entry_get_text(GTK_ENTRY(url_entry))));
}


static GaimPluginPrefFrame *get_grr_pref_frame(GaimPlugin *plugin)
{
    GtkWidget *vbox, *frame;

    GtkTreeViewColumn   *col;
    GtkCellRenderer     *renderer;
    GtkWidget		*scrolled;

    GtkWidget *hbox;
    GtkWidget *button, *label;
    GtkSizeGroup *sg;

    vbox = gtk_vbox_new(FALSE, 18);
    gtk_container_set_border_width(GTK_CONTAINER(vbox), 12);

    frame = gaim_gtk_make_frame(vbox, "Feed list");

    //LIST
    pref_tree = gtk_tree_view_new();

    scrolled = gtk_scrolled_window_new ( NULL, NULL );
    gtk_scrolled_window_set_policy ( GTK_SCROLLED_WINDOW ( scrolled ),
                                    GTK_POLICY_AUTOMATIC,
				    GTK_POLICY_AUTOMATIC);
    gtk_container_add ( GTK_CONTAINER ( scrolled ),
                       GTK_WIDGET ( pref_tree ) );

    gtk_container_add ( GTK_CONTAINER ( frame ),
		       GTK_WIDGET ( scrolled ) );


    gtk_widget_set_size_request(pref_tree, 260,200);

    /* --- Column #1 --- */

    col = gtk_tree_view_column_new();

    gtk_tree_view_column_set_title(col, "Title");

    /* pack tree view column into tree view */
    gtk_tree_view_append_column(GTK_TREE_VIEW(pref_tree), col);

    renderer = gtk_cell_renderer_text_new();

    /* pack cell renderer into tree view column */
    gtk_tree_view_column_pack_start(col, renderer, TRUE);

    /* connect 'text' property of the cell renderer to
     *  model column that contains the first name */
    gtk_tree_view_column_add_attribute(col, renderer, "text", 0);

    g_object_set(renderer,
                 "weight", PANGO_WEIGHT_BOLD,
                 "weight-set", TRUE,
                 NULL);

    /* --- Column #2 --- */

    col = gtk_tree_view_column_new();

    gtk_tree_view_column_set_title(col, "Url");

    /* pack tree view column into tree view */
    gtk_tree_view_append_column(GTK_TREE_VIEW(pref_tree), col);

    renderer = gtk_cell_renderer_text_new();

    /* pack cell renderer into tree view column */
    gtk_tree_view_column_pack_start(col, renderer, TRUE);

    /* connect 'text' property of the cell renderer to
     *  model column that contains the last name */
    gtk_tree_view_column_add_attribute(col, renderer, "text", 1);

    model = create_and_fill_model();

    gtk_tree_view_set_model(GTK_TREE_VIEW(pref_tree), model);

    g_object_unref(model); /* destroy model automatically with view */

    gtk_tree_selection_set_mode(gtk_tree_view_get_selection(GTK_TREE_VIEW(pref_tree)),
                                GTK_SELECTION_SINGLE);

    // /LIST

    // DELETE button

    hbox = gtk_hbutton_box_new();

    gtk_container_add ( GTK_CONTAINER ( frame ),
		       GTK_WIDGET ( hbox ) );

    gtk_widget_show(hbox);

    button = gtk_button_new_from_stock(GTK_STOCK_DELETE);
    g_signal_connect(G_OBJECT(button), "clicked",
		     G_CALLBACK(list_delete), NULL);
    gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
    gtk_widget_set_sensitive(button, FALSE);

    g_signal_connect(G_OBJECT(gtk_tree_view_get_selection(GTK_TREE_VIEW(pref_tree))),
		     "changed", G_CALLBACK(on_selection_changed), button);

    // / DELETE button

    // Add frame

    frame = gaim_gtk_make_frame(vbox, _("Add a new feed"));
    gtk_widget_set_size_request(frame, 300, -1);

    hbox = gtk_hbox_new(FALSE, 2);
    gtk_box_pack_start(GTK_BOX(frame), hbox, FALSE, FALSE, 0);
    gtk_widget_show(hbox);

    sg = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);

    label = gtk_label_new_with_mnemonic(_("Feed title:"));
    gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);

    name_entry = gtk_entry_new();
    gtk_entry_set_max_length(GTK_ENTRY(name_entry), 100);
    gtk_box_pack_start(GTK_BOX(hbox), name_entry, TRUE, TRUE, 0);
    gtk_size_group_add_widget(sg, name_entry);
    gtk_label_set_mnemonic_widget(GTK_LABEL(label), name_entry);
    gtk_widget_show(name_entry);

    hbox = gtk_hbox_new(FALSE, 2);
    gtk_box_pack_start(GTK_BOX(frame), hbox, FALSE, FALSE, 0);
    gtk_widget_show(hbox);

    label = gtk_label_new_with_mnemonic(_("Feed URL:"));
    gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);

    url_entry = gtk_entry_new();
    gtk_entry_set_max_length(GTK_ENTRY(url_entry), 200);
    gtk_box_pack_start(GTK_BOX(hbox), url_entry, TRUE, TRUE, 0);
    gtk_size_group_add_widget(sg, url_entry);
    gtk_label_set_mnemonic_widget(GTK_LABEL(label), url_entry);
    gtk_widget_show(url_entry);

    hbox = gtk_hbutton_box_new();
    gtk_box_pack_start(GTK_BOX(frame), hbox, FALSE, FALSE, 0);
    button = gtk_button_new_from_stock(GTK_STOCK_ADD);
    g_signal_connect(G_OBJECT(button), "clicked",
		     G_CALLBACK(list_add_new), NULL);
    gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
    g_signal_connect(G_OBJECT(name_entry), "changed", G_CALLBACK(on_entry_changed), button);
    g_signal_connect(G_OBJECT(url_entry), "changed", G_CALLBACK(on_entry_changed), button);
    gtk_widget_set_sensitive(button, FALSE);
    gtk_widget_show(button);

    // / Add frame

    frame = gaim_gtk_make_frame(vbox, "Options");

    gaim_gtk_prefs_labeled_spin_button(frame, "Update time",
				       "/plugins/gtk/gaim-rss-reader/update_time",
                                       10, 65535, NULL);

    g_signal_connect(G_OBJECT(gtk_tree_view_get_selection(GTK_TREE_VIEW(pref_tree))),
		     "changed", G_CALLBACK(on_selection_changed), button);

    /*gaim_gtk_prefs_checkbox("Show playlist in the menu",
                            "/plugins/gtk/amc_grim/gaim-xmms-remote/show_playlist",
                            frame);

    gaim_gtk_prefs_checkbox("Show button in IMs",
                            "/plugins/gtk/amc_grim/gaim-xmms-remote/im",
                            frame);

    gaim_gtk_prefs_checkbox("Show button in Chats",
                            "/plugins/gtk/amc_grim/gaim-xmms-remote/chat",
			    frame);
			    */

    gtk_widget_show_all(vbox);

    return vbox;
}




static GaimPluginUiInfo ui_info = {
    get_grr_pref_frame
};

static GaimPluginInfo grr_info = {
	GAIM_PLUGIN_MAGIC,
	GAIM_MAJOR_VERSION,
	GAIM_MINOR_VERSION,							/* api version */
	GAIM_PLUGIN_STANDARD,								/* type */
	GAIM_GTK_PLUGIN_TYPE,								/* ui requirement */
	0,													/* flags */
	NULL,												/* dependencies */
	GAIM_PRIORITY_DEFAULT,								/* priority */

	"Gtk-sasha-Gaim-RSS-Reader",					/* id */
	"Gaim-RSS-Reader",								/* name */
	GRR_VERSION,										/* version */
	"Read RSS",			/* summary */
	"A small plugin that adds a button to the\n"
	"Gaim conversation windows, so that you\n"
	"can read RSS feeds from within Gaim.",			/* description */

	"Alexander Movtchan <sasha@atis-labs.com>",		/* author */
        "http://gaim-rss-reader.sourceforge.net/",	/* homepage */

	grr_load,											/* load */
	grr_unload,											/* unload */
	NULL,												/* destroy */

        &ui_info,											/* ui info */
        NULL,
	NULL,											/* extra info */
	NULL,
	NULL
};

GAIM_INIT_PLUGIN(Gaim-RSS-Reader, init_plugin, grr_info)

